@@ -389,6 +389,7 @@ import traceback
 from copy import deepcopy
 from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Type
 
+from ae.base import NAME_PARTS_SEP, snake_to_camel                  # type: ignore
 from ae.updater import check_all                                    # type: ignore
 from ae.inspector import stack_var                                  # type: ignore
 from ae.files import FilesRegister, RegisteredFile                  # type: ignore
@@ -397,7 +398,7 @@ from ae.core import DEBUG_LEVELS                                    # type: igno
 from ae.console import ConsoleApp                                   # type: ignore
 
 
-__version__ = '0.1.34'
+__version__ = '0.1.37'
 
 
 AppStateType = Dict[str, Any]                   #: app state config variable type
@@ -422,7 +423,6 @@ THEME_LIGHT_FONT_COLOR = COLOR_BLACK            #: light theme font color in rgb
 check_all()     # let ae_updater module check/install any outstanding updates or new app versions
 
 
-FLOW_PARTS_SEP = '_'
 FLOW_KEY_SEP = ':'
 
 FLOW_ACTION_RE = re.compile("[a-z0-9]+")
@@ -445,7 +445,7 @@ def id_of_flow(action: str, obj: str, key: str = "") -> str:
         f"flow action only allows lowercase letters and digits: got '{action}'"
     assert obj == '' or FLOW_OBJECT_RE.fullmatch(obj), \
         f"flow object only allows letters, digits and underscores: got '{obj}'"
-    cid = f'{action}{FLOW_PARTS_SEP if action and obj else ""}{obj}'
+    cid = f'{action}{NAME_PARTS_SEP if action and obj else ""}{obj}'
     if key:
         cid += f'{FLOW_KEY_SEP}{key}'
     return cid
@@ -466,7 +466,7 @@ def flow_action_split(flow_id: str) -> Tuple[str, str]:
     :param flow_id:         flow id.
     :return:                tuple of (flow action string, flow obj and key string)
     """
-    idx = flow_id.find(FLOW_PARTS_SEP)
+    idx = flow_id.find(NAME_PARTS_SEP)
     if idx != -1:
         return flow_id[:idx], flow_id[idx + 1:]
     return flow_id, ""
@@ -513,8 +513,7 @@ def flow_popup_class_name(flow_id: str) -> str:
     """
     flow, _index = flow_key_split(flow_id)
     action, obj = flow_action_split(flow)
-    parts = obj.split(FLOW_PARTS_SEP)
-    return f'{"".join(part.capitalize() for part in parts)}{action.capitalize()}Popup'
+    return f'{snake_to_camel(obj)}{action.capitalize()}Popup'
 
 
 def replace_flow_action(flow_id: str, new_action: str):
@@ -524,7 +523,7 @@ def replace_flow_action(flow_id: str, new_action: str):
     :param new_action:      action to be set/replaced within passed flow id.
     :return:                flow id with new action and object/key from passed flow id.
     """
-    return new_action + FLOW_PARTS_SEP + flow_action_split(flow_id)[1]
+    return new_action + NAME_PARTS_SEP + flow_action_split(flow_id)[1]
 
 
 class MainAppBase(ConsoleApp, ABC):
@@ -612,10 +611,10 @@ class MainAppBase(ConsoleApp, ABC):
         setattr(self, name, value)
         if self.framework_app:
             if is_app_state:
-                if hasattr(self.framework_app, 'ae_states'):        # has observable DictProperty duplicates
+                if hasattr(self.framework_app, 'app_states'):       # has observable DictProperty duplicates
                     # noinspection PyUnresolvedReferences
-                    self.framework_app.ae_states[name] = value
-                name = 'ae_state_' + name
+                    self.framework_app.app_states[name] = value
+                name = 'app_state_' + name
             if hasattr(self.framework_app, name):                   # has observable attribute duplicate
                 setattr(self.framework_app, name, value)
 
@@ -861,12 +860,30 @@ class MainAppBase(ConsoleApp, ABC):
 
         return True
 
-    def flow_path_action(self, path_index: int = -1) -> str:
-        """ determine the action of the last/newest entry in the flow_path. """
-        if len(self.flow_path) >= (abs(path_index) if path_index < 0 else path_index + 1):
-            return flow_action(self.flow_path[path_index])
+    def flow_path_action(self, flow_path: Optional[List[str]] = None, path_index: int = -1) -> str:
+        """ determine the action of the last/newest entry in the flow_path.
+
+        :param flow_path:       flow path to get the flow action from (default=self.flow_path).
+        :param path_index:      index in the flow_path.
+        :return:                flow action string or empty string if index does not exist.
+        """
+        if flow_path is None:
+            flow_path = self.flow_path
+        if len(flow_path) >= (abs(path_index) if path_index < 0 else path_index + 1):
+            return flow_action(flow_path[path_index])
         return ''
 
+    def flow_path_strip(self, flow_path: List[str]) -> List[str]:
+        """ return flow_path copy with all non-enter actions stripped from the end.
+
+        :param flow_path:       flow path list to strip.
+        :return:                stripped flow path copy.
+        """
+        deep = len(flow_path)
+        while deep and self.flow_path_action(flow_path=flow_path, path_index=deep - 1) != 'enter':
+            deep -= 1
+        return flow_path[:deep]
+
     def on_flow_change(self, flow_id: str, event_kwargs: Dict[str, Any]) -> bool:
         """ checking if exists a Popup class for the new flow and if yes then open it.
 
@@ -1037,12 +1054,12 @@ class MainAppBase(ConsoleApp, ABC):
         :return:                widget that has the specified attribute with the specified value.
         """
         def child_wid(children):
-            """ search children for widget.ae_flow_id attr value equal to :paramref:`~widget_by_flow_id.flow_id`. """
+            """ bottom up search within children for a widget with matching attribute name and value. """
             for widget in children:
                 found = child_wid(widget.children)
                 if found:
                     return found
-                elif getattr(widget, att_name, None) == att_value:
+                if getattr(widget, att_name, None) == att_value:
                     return widget
             return None
 
@@ -1051,18 +1068,18 @@ class MainAppBase(ConsoleApp, ABC):
     def widget_by_flow_id(self, flow_id: str) -> Optional[Any]:
         """ determine the widget having the passed flow_id.
 
-        :param flow_id:         flow id value of the searched widget's `ae_flow_id` attribute.
-        :return:                widget that has a `ae_flow_id` attribute with the value of the passed flow id.
+        :param flow_id:         flow id value of the searched widget's `tap_flow_id` attribute.
+        :return:                widget that has a `tap_flow_id` attribute with the value of the passed flow id.
         """
-        return self.widget_by_attribute('ae_flow_id', flow_id)
+        return self.widget_by_attribute('tap_flow_id', flow_id)
 
     def widget_by_state_name(self, state_name: str) -> Optional[Any]:
-        """ determine the widget having the passed app state name (ae_state_name).
+        """ determine the widget having the passed app state name (app_state_name).
 
-        :param state_name:      app state name of the widget's `ae_state_name` attribute.
-        :return:                widget that has a `ae_state_name` attribute with the passed app state name.
+        :param state_name:      app state name of the widget's `app_state_name` attribute.
+        :return:                widget that has a `app_state_name` attribute with the passed app state name.
         """
-        return self.widget_by_attribute('ae_state_name', state_name)
+        return self.widget_by_attribute('app_state_name', state_name)
 
     def win_pos_size_change(self, *win_pos_size):
         """ screen resize handler (called on window resize or when app will exit/stop via closed event.
